@@ -14,7 +14,7 @@ from inspect import (
 from io import IOBase
 from os import PathLike
 from queue import Queue
-from socket import AddressFamily, SocketKind, SocketType
+from socket import AddressFamily, SocketKind
 from threading import Thread
 from types import TracebackType
 from typing import (
@@ -53,7 +53,7 @@ else:
     _T = TypeVar('_T')
 
     def _get_task_callbacks(task: asyncio.Task) -> Iterable[Callable]:
-        return task._callbacks  # type: ignore
+        return task._callbacks
 
     def native_run(main, *, debug=False):
         # Snatched from Python 3.7
@@ -136,21 +136,18 @@ _root_task: RunVar[Optional[asyncio.Task]] = RunVar('_root_task')
 
 
 def find_root_task() -> asyncio.Task:
-    try:
-        root_task = _root_task.get()
-    except LookupError:
-        for task in all_tasks():
-            if task._callbacks:
-                for cb in _get_task_callbacks(task):
-                    if (cb is _run_until_complete_cb
-                            or getattr(cb, '__module__', None) == 'uvloop.loop'):
-                        _root_task.set(task)
-                        return task
+    root_task = _root_task.get(None)
+    if root_task is not None and not root_task.done():
+        return root_task
 
-        _root_task.set(None)
-    else:
-        if root_task is not None:
-            return root_task
+    # Look for a task that has been started via run_until_complete()
+    for task in all_tasks():
+        if task._callbacks and not task.done():
+            for cb in _get_task_callbacks(task):
+                if (cb is _run_until_complete_cb
+                        or getattr(cb, '__module__', None) == 'uvloop.loop'):
+                    _root_task.set(task)
+                    return task
 
     # Look up the topmost task in the AnyIO task tree, if possible
     task = cast(asyncio.Task, current_task())
@@ -214,7 +211,7 @@ def _maybe_set_event_loop_policy(policy: Optional[asyncio.AbstractEventLoopPolic
 
 
 def run(func: Callable[..., Awaitable[T_Retval]], *args: object,
-        debug: bool = False, use_uvloop: bool = True,
+        debug: bool = False, use_uvloop: bool = False,
         policy: Optional[asyncio.AbstractEventLoopPolicy] = None) -> T_Retval:
     @wraps(func)
     async def wrapper() -> T_Retval:
@@ -519,7 +516,10 @@ class _AsyncioTaskStatus(abc.TaskStatus):
         self._future = future
 
     def started(self, value: object = None) -> None:
-        self._future.set_result(value)
+        try:
+            self._future.set_result(value)
+        except asyncio.InvalidStateError:
+            raise RuntimeError("called 'started' twice on the same task status") from None
 
 
 class TaskGroup(abc.TaskGroup):
@@ -712,6 +712,16 @@ class WorkerThread(Thread):
         self.queue: Queue[Union[Tuple[Callable, tuple, asyncio.Future], None]] = Queue(2)
         self.idle_since = current_time()
 
+    def _report_result(self, future: asyncio.Future, result: Any,
+                       exc: Optional[BaseException]) -> None:
+        self.idle_since = current_time()
+        self.idle_workers.append(self)
+        if not future.cancelled():
+            if exc is not None:
+                future.set_exception(exc)
+            else:
+                future.set_result(result)
+
     def run(self) -> None:
         with claim_worker_thread('asyncio'):
             threadlocals.loop = self.loop
@@ -723,14 +733,16 @@ class WorkerThread(Thread):
 
                 func, args, future = item
                 if not future.cancelled():
+                    result = None
+                    exception: Optional[BaseException] = None
                     try:
                         result = func(*args)
                     except BaseException as exc:
-                        if not self.loop.is_closed() and not future.cancelled():
-                            self.loop.call_soon_threadsafe(future.set_exception, exc)
-                    else:
-                        if not self.loop.is_closed() and not future.cancelled():
-                            self.loop.call_soon_threadsafe(future.set_result, result)
+                        exception = exc
+
+                    if not self.loop.is_closed():
+                        self.loop.call_soon_threadsafe(
+                            self._report_result, future, result, exception)
 
                 self.queue.task_done()
 
@@ -785,11 +797,7 @@ async def run_sync_in_worker_thread(
                     worker.stop()
 
             worker.queue.put_nowait((func, args, future))
-            try:
-                return await future
-            finally:
-                worker.idle_since = current_time()
-                idle_workers.append(worker)
+            return await future
 
 
 def run_sync_from_thread(func: Callable[..., T_Retval], *args: object,
@@ -1139,14 +1147,14 @@ class UNIXSocketStream(abc.SocketStream):
     _send_future: Optional[asyncio.Future] = None
     _closing = False
 
-    def __init__(self, raw_socket: socket.SocketType):
+    def __init__(self, raw_socket: socket.socket):
         self.__raw_socket = raw_socket
         self._loop = get_running_loop()
         self._receive_guard = ResourceGuard('reading from')
         self._send_guard = ResourceGuard('writing to')
 
     @property
-    def _raw_socket(self) -> SocketType:
+    def _raw_socket(self) -> socket.socket:
         return self.__raw_socket
 
     def _wait_until_readable(self, loop: asyncio.AbstractEventLoop) -> asyncio.Future:
@@ -1240,7 +1248,7 @@ class UNIXSocketStream(abc.SocketStream):
 
         for cmsg_level, cmsg_type, cmsg_data in ancdata:
             if cmsg_level != socket.SOL_SOCKET or cmsg_type != socket.SCM_RIGHTS:
-                raise RuntimeError(f'Received unexpected ancillary data; message = {message}, '
+                raise RuntimeError(f'Received unexpected ancillary data; message = {message!r}, '
                                    f'cmsg_level = {cmsg_level}, cmsg_type = {cmsg_type}')
 
             fds.frombytes(cmsg_data[:len(cmsg_data) - (len(cmsg_data) % fds.itemsize)])
@@ -1266,8 +1274,12 @@ class UNIXSocketStream(abc.SocketStream):
         with self._send_guard:
             while True:
                 try:
-                    self.__raw_socket.sendmsg([message],
-                                              [(socket.SOL_SOCKET, socket.SCM_RIGHTS, fdarray)])
+                    # The ignore can be removed after mypy picks up
+                    # https://github.com/python/typeshed/pull/5545
+                    self.__raw_socket.sendmsg(
+                        [message],
+                        [(socket.SOL_SOCKET, socket.SCM_RIGHTS, fdarray)]  # type: ignore
+                    )
                     break
                 except BlockingIOError:
                     await self._wait_until_writable(loop)
@@ -1293,7 +1305,7 @@ class TCPSocketListener(abc.SocketListener):
     _accept_scope: Optional[CancelScope] = None
     _closed = False
 
-    def __init__(self, raw_socket: socket.SocketType):
+    def __init__(self, raw_socket: socket.socket):
         self.__raw_socket = raw_socket
         self._loop = cast(asyncio.BaseEventLoop, get_running_loop())
         self._accept_guard = ResourceGuard('accepting connections from')
@@ -1348,7 +1360,7 @@ class TCPSocketListener(abc.SocketListener):
 
 
 class UNIXSocketListener(abc.SocketListener):
-    def __init__(self, raw_socket: socket.SocketType):
+    def __init__(self, raw_socket: socket.socket):
         self.__raw_socket = raw_socket
         self._loop = get_running_loop()
         self._accept_guard = ResourceGuard('accepting connections from')
@@ -1377,7 +1389,7 @@ class UNIXSocketListener(abc.SocketListener):
         self.__raw_socket.close()
 
     @property
-    def _raw_socket(self) -> SocketType:
+    def _raw_socket(self) -> socket.socket:
         return self.__raw_socket
 
 
@@ -1390,7 +1402,7 @@ class UDPSocket(abc.UDPSocket):
         self._closed = False
 
     @property
-    def _raw_socket(self) -> SocketType:
+    def _raw_socket(self) -> socket.socket:
         return self._transport.get_extra_info('socket')
 
     async def aclose(self) -> None:
@@ -1436,7 +1448,7 @@ class ConnectedUDPSocket(abc.ConnectedUDPSocket):
         self._closed = False
 
     @property
-    def _raw_socket(self) -> SocketType:
+    def _raw_socket(self) -> socket.socket:
         return self._transport.get_extra_info('socket')
 
     async def aclose(self) -> None:
@@ -1499,6 +1511,9 @@ async def connect_unix(path: str) -> UNIXSocketStream:
             loop.add_writer(raw_socket, f.set_result, None)
             f.add_done_callback(lambda _: loop.remove_writer(raw_socket))
             await f
+        except BaseException:
+            raw_socket.close()
+            raise
         else:
             return UNIXSocketStream(raw_socket)
 
@@ -1534,16 +1549,14 @@ async def getaddrinfo(host: Union[bytearray, bytes, str], port: Union[str, int,
 
 
 async def getnameinfo(sockaddr: IPSockAddrType, flags: int = 0) -> Tuple[str, str]:
-    # https://github.com/python/typeshed/pull/4305
-    result = await get_running_loop().getnameinfo(sockaddr, flags)
-    return cast(Tuple[str, str], result)
+    return await get_running_loop().getnameinfo(sockaddr, flags)
 
 
 _read_events: RunVar[Dict[Any, asyncio.Event]] = RunVar('read_events')
 _write_events: RunVar[Dict[Any, asyncio.Event]] = RunVar('write_events')
 
 
-async def wait_socket_readable(sock: socket.SocketType) -> None:
+async def wait_socket_readable(sock: socket.socket) -> None:
     await checkpoint()
     try:
         read_events = _read_events.get()
@@ -1570,7 +1583,7 @@ async def wait_socket_readable(sock: socket.SocketType) -> None:
         raise ClosedResourceError
 
 
-async def wait_socket_writable(sock: socket.SocketType) -> None:
+async def wait_socket_writable(sock: socket.socket) -> None:
     await checkpoint()
     try:
         write_events = _write_events.get()
@@ -1746,7 +1759,7 @@ def current_default_thread_limiter() -> CapacityLimiter:
 # Operating system signals
 #
 
-class _SignalReceiver(DeprecatedAsyncContextManager):
+class _SignalReceiver(DeprecatedAsyncContextManager["_SignalReceiver"]):
     def __init__(self, signals: Tuple[int, ...]):
         self._signals = signals
         self._loop = get_running_loop()
@@ -1796,7 +1809,7 @@ def open_signal_receiver(*signals: int) -> _SignalReceiver:
 def _create_task_info(task: asyncio.Task) -> TaskInfo:
     task_state = _task_states.get(task)
     if task_state is None:
-        name = task.get_name() if _native_task_names else None  # type: ignore
+        name = task.get_name() if _native_task_names else None
         parent_id = None
     else:
         name = task_state.name
@@ -1844,8 +1857,7 @@ class TestRunner(abc.TestRunner):
         for task in to_cancel:
             task.cancel()
 
-        self._loop.run_until_complete(
-            asyncio.gather(*to_cancel, loop=self._loop, return_exceptions=True))
+        self._loop.run_until_complete(asyncio.gather(*to_cancel, return_exceptions=True))
 
         for task in to_cancel:
             if task.cancelled():
